                     GEM++ - C++ LIBRARIES FOR GEM/AES/VDI



NAME
     GEMtimer - Timer event handlers

DESCRIPTION
     GEM allows an application to wait for an amount of time to pass.

     This class introduces an additional scheduling layer that 
     allows multiple timer events to be pending.

     This is an abstract base class, since Expire() is undefined.

CONSTRUCTORS
     GEMtimer(GEMactivity& in, int millisec)
       Create a timer which will be called regularly at the given interval
       of time.  The actual interval may be arbitrarilly inaccurate, but
       GEMtimers waiting for intervals A and B *will* expire in the order
       of A and B.

METHODS
     int Interval()
       Returns the interval of the timer.

     void Interval(int i)
       Set the timer interval.  ONLY TO BE CALLED DURING Expire().

     virtual GEMfeedback Expire(const GEMevent&)=0
       This method should carry out whatever activities are required,
       possibly call Interval(int), and return a GEMfeedback.

     static int NextInterval()
     static GEMfeedback ExpireNext(const GEMevent&)
       These static methods are for the service provider (GEMactivity).

EXAMPLES
     See "Clock" in example.cc

SEE ALSO
     GEMevent, GEMactivity

BUGS
     Bugs in GEM++ should be reported to warwick@cs.uq.oz.au

AUTHOR
     Warwick Allison, 1993.
     warwick@cs.uq.oz.au

COPYING
     This functionality is part of the GEM++ library,
     and is Copyright 1993 by Warwick W. Allison.

     GEM++ is free and protected under the GNU Library General Public
     License.

     You are free to copy and modify these sources, provided you
     acknowledge the origin by retaining this notice, and adhere to
     the conditions described in the GNU LGPL.
