IMPLEMENTATION MODULE CTYPE;
(*__NO_CHECKS__*)
(*****************************************************************************)
(* 14-Apr-92, Holger Kleinschmidt                                            *)
(* --------------------------------------------------------------------------*)
(* STATUS: OK                                                                *)
(*****************************************************************************)

VAL_INTRINSIC

TYPE
  CharTypes = (LOWER, UPPER, DIGIT, ODIGIT, XDIGIT, PUNCT, CONTROL, SPACE);

  CharType  = PACKEDSET OF CharTypes;

CONST
  NOALPHA = CharType{DIGIT..SPACE};
  NOALNUM = CharType{PUNCT..SPACE};
  GRAPH   = CharType{LOWER..PUNCT};

VAR
  ctype : ARRAY [0C..377C] OF CharType;

(*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*)

PROCEDURE ISALNUM ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(NOT(ctype[c] <= NOALNUM));
END ISALNUM;

(*---------------------------------------------------------------------------*)

PROCEDURE ISALPHA ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(NOT(ctype[c] <= NOALPHA));
END ISALPHA;

(*---------------------------------------------------------------------------*)

PROCEDURE ISASCII ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN( c <= 177C );
END ISASCII;

(*---------------------------------------------------------------------------*)

PROCEDURE ISCNTRL ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(CONTROL IN ctype[c]);
END ISCNTRL;

(*---------------------------------------------------------------------------*)

PROCEDURE ISDIGIT ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(DIGIT IN ctype[c]);
END ISDIGIT;

(*---------------------------------------------------------------------------*)

PROCEDURE ISGRAPH ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(ctype[c] <= GRAPH);
END ISGRAPH;

(*---------------------------------------------------------------------------*)

PROCEDURE ISLOWER ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(LOWER IN ctype[c]);
END ISLOWER;

(*---------------------------------------------------------------------------*)

PROCEDURE ISODIGIT ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(ODIGIT IN ctype[c]);
END ISODIGIT;

(*---------------------------------------------------------------------------*)

PROCEDURE ISPRINT ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(NOT(CONTROL IN ctype[c]));
END ISPRINT;

(*---------------------------------------------------------------------------*)

PROCEDURE ISPUNCT ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(PUNCT IN ctype[c]);
END ISPUNCT;

(*---------------------------------------------------------------------------*)

PROCEDURE ISSPACE ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(SPACE IN ctype[c]);
END ISSPACE;

(*---------------------------------------------------------------------------*)

PROCEDURE ISUPPER ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(UPPER IN ctype[c]);
END ISUPPER;

(*---------------------------------------------------------------------------*)

PROCEDURE ISXDIGIT ((* EIN/ -- *) c : CHAR ): BOOLEAN;
(*T*)
BEGIN
 RETURN(XDIGIT IN ctype[c]);
END ISXDIGIT;

(*---------------------------------------------------------------------------*)

PROCEDURE TOLOWER ((* EIN/ -- *) c : CHAR ): CHAR;
(*T*)
BEGIN
 IF ('A' <= c) AND (c <= 'Z')  THEN
   RETURN(CHR(ORD(c) + 32));
 ELSE
   RETURN(c);
 END;
END TOLOWER;

(*---------------------------------------------------------------------------*)

PROCEDURE TOUPPER ((* EIN/ -- *) c : CHAR ): CHAR;
(*T*)
BEGIN
 IF ('a' <= c) AND (c <= 'z')  THEN
   RETURN(CHR(ORD(c) - 32));
 ELSE
   RETURN(c);
 END;
END TOUPPER;

(*---------------------------------------------------------------------------*)

PROCEDURE TOCARD ((* EIN/ -- *) c : CHAR ): CARDINAL;
(*T*)
BEGIN
 IF c <= '9'  THEN
   RETURN(VAL(CARDINAL,ORD(c) - ORD('0')));
 ELSE
   RETURN(VAL(CARDINAL,ORD(TOUPPER(c)) - ORD('A') + 10));
 END;
END TOCARD;

(*---------------------------------------------------------------------------*)

PROCEDURE TODIGIT ((* EIN/ -- *) i : CARDINAL ): CHAR;
(*T*)
BEGIN
 IF i <= 9 THEN
   RETURN(CHR(i + VAL(CARDINAL,ORD('0'))));
 ELSE
   RETURN(CHR(i + VAL(CARDINAL,ORD('A')) - 10));
 END;
END TODIGIT;

(*===========================================================================*)

VAR c : CHAR;

BEGIN (* CTYPE *)
 FOR c:=0C TO 37C DO
   ctype[c] := CharType{CONTROL};
 END;
 ctype[177C] := CharType{CONTROL};
 FOR c:=11C TO 15C DO
   ctype[c] := CharType{CONTROL, SPACE};
 END;
 ctype[40C] := CharType{SPACE};
 FOR c:=41C TO 57C DO
   ctype[c] := CharType{PUNCT};
 END;
 FOR c:=41C TO 57C DO
   ctype[c] := CharType{PUNCT};
 END;
 FOR c:=72C TO 100C DO
   ctype[c] := CharType{PUNCT};
 END;
 FOR c:=133C TO 140C DO
   ctype[c] := CharType{PUNCT};
 END;
 FOR c:=173C TO 176C DO
   ctype[c] := CharType{PUNCT};
 END;
 FOR c:=60C TO 67C DO
   ctype[c] := CharType{DIGIT, ODIGIT, XDIGIT};
 END;
 ctype[70C] := CharType{DIGIT, XDIGIT};
 ctype[71C] := CharType{DIGIT, XDIGIT};
 FOR c:=101C TO 106C DO
   ctype[c] := CharType{XDIGIT, UPPER};
 END;
 FOR c:=107C TO 132C DO
   ctype[c] := CharType{UPPER};
 END;
 FOR c:=141C TO 146C DO
   ctype[c] := CharType{XDIGIT, LOWER};
 END;
 FOR c:=147C TO 172C DO
   ctype[c] := CharType{LOWER};
 END;
(* ab hier evtl. europaeische Buchstaben beruecksichtigen *)
 FOR c:=200C TO 377C DO
   ctype[c] := CharType{};
 END;
END CTYPE.
