IMPLEMENTATION MODULE dir;
(*__NO_CHECKS__*)
(*****************************************************************************)
(* Basiert auf der MiNTLIB von Eric R. Smith                                 *)
(*---------------------------------------------------------------------------*)
(* STATUS: IN ARBEIT                                                         *)
(*---------------------------------------------------------------------------*)
(* 12-Feb-93, Holger Kleinschmidt                                            *)
(*****************************************************************************)

VAL_INTRINSIC
CAST_IMPORT
OSCALL_IMPORT

FROM SYSTEM IMPORT
(* PROC *) ADR;

FROM types IMPORT
(* CONST*) DDIRSEP, NULL, PATHMAX,
(* TYPE *) SIGNEDWORD, UNSIGNEDWORD, UNSIGNEDLONG, FileName, PathName;

FROM CTYPE IMPORT
(* PROC *) TOCARD;

FROM pSTRING IMPORT
(* CONST*) EOS,
(* TYPE *) StrPtr, StrRange,
(* PROC *) SLEN, ASSIGN, APPEND, EQUAL, AssignM2ToC;

FROM err IMPORT
(* CONST*) eOK, eFILNF, eNMFIL, eRANGE, eACCDN,
           ENOMEM, EEXIST, ENOSYS, ERANGE, EBADF, ENOENT, ENOTDIR, ENOTEMPTY,
(* VAR  *) errno;

FROM DosFile IMPORT
(* CONST*) FINDALL,
(* TYPE *) DTA, FileAttributes, FileAttribute,
(* PROC *) CompletePath, DosToUnix, UnixToDos, FindFirst, FindNext;
#if MINT
FROM DosSystem IMPORT MiNTVersion;
#endif

FROM file IMPORT
(* CONST*) fOK,
(* TYPE *) modeT,
(* PROC *) access;

FROM Storage IMPORT
(* PROC *) ALLOCATE, DEALLOCATE;

#include "oscalls.m2h"

(*==========================================================================*)

TYPE
  DIRPtr = POINTER TO DIRType;

  DIRState = (STARTSEARCH, INSEARCH, NMFILE);
  DIRType = RECORD
    status  : DIRState;
    dta     : DTA;
    buf     : DirentRec;
    dirname : PathName;
  END;

#if MINT
VAR
  MiNT : CARDINAL;
#endif

(*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*)

PROCEDURE mkdir ((* EIN/ -- *) REF dir  : ARRAY OF CHAR;
                 (* EIN/ -- *)     mode : modeT         ): INTEGER;
(*T*)
VAR wres  : SIGNEDWORD;
    dot   : BOOLEAN;
    done  : BOOLEAN;
    path0 : PathName;

BEGIN
 UnixToDos(dir, path0, dot, done);
 IF NOT done THEN
   RETURN(-1);
 END;

 IF access(dir, fOK) = 0 THEN
   errno := EEXIST;
   RETURN(-1);
 ELSIF (errno <> ENOENT) AND (errno <> ENOTDIR) THEN
   RETURN(-1);
 END;

 Dcreate(ADR(path0), wres);
 IF wres < eOK THEN
   errno := INT(wres);
   RETURN(-1);
 END;
#if MINT
 IF MiNT >= 9 THEN
   Fchmod(ADR(path0), CAST(UNSIGNEDWORD,mode), wres);
 END;
#endif
 RETURN(0);
END mkdir;

(*---------------------------------------------------------------------------*)

PROCEDURE rmdir ((* EIN/ -- *) REF dir : ARRAY OF CHAR ): INTEGER;
(*T*)
VAR wres  : SIGNEDWORD;
    dot   : BOOLEAN;
    done  : BOOLEAN;
    path0 : PathName;

BEGIN
 UnixToDos(dir, path0, dot, done);
 IF NOT done THEN
   RETURN(-1);
 END;

 Ddelete(ADR(path0), wres);
 IF wres < eOK THEN
   IF wres = eACCDN THEN
     errno := ENOTEMPTY;
   ELSE
     errno := INT(wres);
   END;
   RETURN(-1);
 ELSE
   RETURN(0);
 END;
END rmdir;

(*---------------------------------------------------------------------------*)

PROCEDURE chdir ((* EIN/ -- *) REF dir : ARRAY OF CHAR ): INTEGER;
(*T*)
VAR old   : UNSIGNEDWORD;
    wres  : SIGNEDWORD;
    drvs  : UNSIGNEDLONG;
    dot   : BOOLEAN;
    done  : BOOLEAN;
    c     : CHAR;
    start : CARDINAL;
    path0 : PathName;

BEGIN
 UnixToDos(dir, path0, dot, done);
 IF NOT done THEN
   RETURN(-1);
 END;

 (* aktuelles Laufwerk merken, fuer Fehlerfall *)
 Dgetdrv(old);

 start := 0;
 IF path0[0] = 0C THEN
   path0[0] := DDIRSEP;
   path0[1] := 0C;
 ELSIF path0[1] = ':' THEN
   wres := VAL(SIGNEDWORD,TOCARD(path0[0]) - 10);
   (* neues Laufwerk setzen *)
   Dsetdrv(wres, drvs);
   start := 2;
 END;

 (* Pfad ohne Laufwerksangabe setzen *)
 Dsetpath(ADR(path0[start]), wres);
 IF wres = eOK THEN
   RETURN(0);
 ELSE
   errno := INT(wres);
   Dsetdrv(old, drvs);
   RETURN(-1);
 END;
END chdir;

(*---------------------------------------------------------------------------*)

PROCEDURE getcwd ((* EIN/ -- *) buf    : StrPtr;
                  (* EIN/ -- *) bufsiz : StrRange ): StrPtr;
(*T*)
VAR err   : INTEGER;
    len   : CARDINAL;
    str1  : ARRAY [0..0] OF CHAR;
    path0 : PathName;

BEGIN
 str1[0] := EOS;
 IF CompletePath(str1, path0, len, err) THEN
   DosToUnix(path0, path0, len);
   IF len < VAL(CARDINAL,bufsiz) THEN
     AssignM2ToC(path0, bufsiz, buf);
     RETURN(buf);
   ELSE
     errno := ERANGE;
     RETURN(NULL);
   END;
 ELSIF err = eRANGE THEN
   errno := ERANGE;
 ELSE
   errno := err;
 END;
 RETURN(NULL);
END getcwd;

(*--------------------------------------------------------------------------*)

PROCEDURE opendir ((* EIN/ -- *) REF dir : ARRAY OF CHAR ): DIR;
(*T*)
VAR dirp    : DIRPtr;
    err     : INTEGER;
    lenDir  : CARDINAL;
    dot     : BOOLEAN;
    done    : BOOLEAN;
    drive   : ARRAY [0..1] OF CHAR;
    path0   : PathName;
#if MINT
    dirhndl : UNSIGNEDLONG;
#endif

BEGIN
 UnixToDos(dir, path0, dot, done);
 IF NOT done THEN
   RETURN(NULL);
 END;

#if MINT
 IF MiNT >= 9 THEN
   Dopendir(ADR(path0), 0, dirhndl);
   IF dirhndl >= CAST(UNSIGNEDLONG,0FF000000H) THEN
     errno := INT(dirhndl);
     RETURN(NULL);
   END;
   RETURN(CAST(DIR,dirhndl));
 END;
#endif

 NEW(dirp);
 IF dirp = NULL THEN
   errno := ENOMEM; (* vielleicht bessser: EMFILE ? *)
   RETURN(NULL);
 END;

 WITH dirp^ DO
   (* vollstaendigen Pfad mit Laufwerksangabe merken, damit bei
    * "rewinddir()" das richtige Verzeichnis benutzt wird, falls
    * sich das aktuelle Verzeichnis inzwischen aendert.
    *)
   IF NOT CompletePath(path0, dirname, lenDir, err) THEN
     DISPOSE(dirp);
     errno := err;
     RETURN(NULL);
   END;

   (* alle Dateien finden *)
   IF dirname[lenDir-1] = DDIRSEP THEN
     APPEND("*.*", dirname);
   ELSE
     APPEND("\*.*", dirname);
   END;

   IF FindFirst(dirname, FINDALL, dta, err) THEN
     status := STARTSEARCH;
   ELSIF err = eFILNF THEN
     status := NMFILE;
   ELSE
     DISPOSE(dirp);
     errno := err;
     RETURN(NULL);
   END;
 END; (* WITH dirp^ *)
 RETURN(dirp);
END opendir;

(*---------------------------------------------------------------------------*)

PROCEDURE readdir ((* EIN/ -- *) dirp : DIR ): DirentPtr;
(*T*)
VAR nextfile : DirentPtr;
    err      : INTEGER;
    dir      : DIRPtr;
#if MINT
TYPE
  DirentMiNTType = RECORD
    dIno    : UNSIGNEDLONG;
    dOff    : UNSIGNEDLONG;
    dReclen : UNSIGNEDWORD;
    dName   : FileName;
  END;

  DIRMiNT = POINTER TO DIRMiNTType;

  DIRMiNTType = RECORD
    status  : UNSIGNEDWORD;
    dta     : DTA;
    dirname : POINTER TO PathName;
    buf     : DirentMiNTType;
  END;

VAR
  wres  : SIGNEDWORD;
  dirpm : DIRMiNT;
  dbuf  : RECORD
            ino  : UNSIGNEDLONG;
            name : FileName;
          END;
#endif

BEGIN
 IF dirp = NULL THEN
   errno := EBADF;
   RETURN(NULL);
 END;
#if MINT
 IF MiNT >= 9 THEN
   dirpm := CAST(DIRMiNT,dirp);
   Dreaddir(VAL(UNSIGNEDWORD,SIZE(dbuf)), dirpm, ADR(dbuf), wres);
   IF wres <> eOK THEN
     IF wres <> eNMFIL THEN
       errno := INT(wres);
     END;
     RETURN(NULL);
   END;
   nextfile := CAST(DirentPtr,ADR(dirpm^.buf.dName));
   nextfile^.dName := dbuf.name;
   RETURN(nextfile);
 END;
#endif

 dir := CAST(DIRPtr,dirp);
 WITH dir^ DO
   IF status = NMFILE THEN
     RETURN(NULL);
   ELSIF status = STARTSEARCH THEN
     status := INSEARCH;
   ELSE
     IF NOT FindNext(dta, err) THEN
       IF err = eNMFIL THEN
         status := NMFILE;
       ELSE
         errno := err;
       END;
       RETURN(NULL);
     END;
   END;
 END; (* WITH dir *)

 nextfile := ADR(dir^.buf);
 ASSIGN(dir^.dta.name, nextfile^.dName); (* besser "DosToUnix()" ? *)
 RETURN(nextfile);
END readdir;

(*---------------------------------------------------------------------------*)

PROCEDURE rewinddir ((* EIN/ -- *) dirp : DIR );
(*T*)
VAR err  : INTEGER;
    dir  : DIRPtr;
#if MINT
    wres : SIGNEDWORD;
#endif
BEGIN
 IF dirp = NULL THEN RETURN; END;
#if MINT
 IF MiNT >= 9 THEN
   Drewinddir(dirp, wres);
   RETURN;
 END;
#endif
 dir := CAST(DIRPtr,dirp);
 WITH dir^ DO
   IF FindFirst(dirname, FINDALL, dta, err) THEN
     status := STARTSEARCH;
   ELSE
     status := NMFILE;
   END;
 END;
END rewinddir;

(*---------------------------------------------------------------------------*)

PROCEDURE closedir ((* EIN/AUS *) VAR dirp : DIR ): INTEGER;
(*T*)
VAR dir  : DIRPtr;
#if MINT
    wres : SIGNEDWORD;
#endif
BEGIN
 IF dirp = NULL THEN
   errno := EBADF;
   RETURN(-1);
 END;
 dir := CAST(DIRPtr,dirp);
#if MINT
 IF MiNT >= 9 THEN
   Dclosedir(dir, wres);
   IF wres < eOK THEN
     errno := INT(wres);
     RETURN(-1);
   ELSE
     dirp := NULL;
     RETURN(0);
   END;
 END;
#endif
 DISPOSE(dir);
 dirp := NULL;
 RETURN(0);
END closedir;

(*--------------------------------------------------------------------------*)

PROCEDURE unlink ((* EIN/ -- *) REF file : ARRAY OF CHAR ): INTEGER;
(*T*)
VAR wres  : SIGNEDWORD;
    done  : BOOLEAN;
    dot   : BOOLEAN;
    path0 : PathName;

BEGIN
 UnixToDos(file, path0, dot, done);
 IF NOT done THEN
   RETURN(-1);
 END;

 Fdelete(ADR(path0), wres);
 IF wres < eOK THEN
   errno := INT(wres);
   RETURN(-1);
 ELSE
   RETURN(0);
 END;
END unlink;

(*--------------------------------------------------------------------------*)

PROCEDURE link ((* EIN/ -- *) REF old : ARRAY OF CHAR;
                (* EIN/ -- *) REF new : ARRAY OF CHAR ): INTEGER;
(*T*)
#if MINT
VAR dot    : BOOLEAN;
    done1  : BOOLEAN;
    done2  : BOOLEAN;
    wres   : SIGNEDWORD;
    path01 : PathName;
    path02 : PathName;
#endif
BEGIN
#if MINT
 IF MiNT >= 9 THEN
   UnixToDos(old, path01, dot, done1);
   UnixToDos(new, path02, dot, done2);
   IF NOT (done1 AND done2) THEN
     RETURN(-1);
   END;

   Flink(ADR(path01), ADR(path02), wres);
   IF wres < eOK THEN
     errno := INT(wres);
     RETURN(-1);
   ELSE
     RETURN(0);
   END;
 END;
#endif
 errno := ENOSYS;
 RETURN(-1);
END link;

(*--------------------------------------------------------------------------*)

PROCEDURE symlink ((* EIN/ -- *) REF old : ARRAY OF CHAR;
                   (* EIN/ -- *) REF new : ARRAY OF CHAR ): INTEGER;
(*T*)
#if MINT
VAR dot    : BOOLEAN;
    done1  : BOOLEAN;
    done2  : BOOLEAN;
    wres   : SIGNEDWORD;
    path01 : PathName;
    path02 : PathName;
#endif
BEGIN
#if MINT
 IF MiNT >= 9 THEN
   UnixToDos(old, path01, dot, done1);
   UnixToDos(new, path02, dot, done2);
   IF NOT (done1 AND done2) THEN
     RETURN(-1);
   END;

   Fsymlink(ADR(path01), ADR(path02), wres);
   IF wres < eOK THEN
     errno := INT(wres);
     RETURN(-1);
   ELSE
     RETURN(0);
   END;
 END;
#endif
 errno := ENOSYS;
 RETURN(-1);
END symlink;

(*--------------------------------------------------------------------------*)

PROCEDURE readlink ((* EIN/ -- *) REF lname  : ARRAY OF CHAR;
                    (* EIN/ -- *)     buf    : StrPtr;
                    (* EIN/ -- *)     bufsiz : StrRange      ): INTEGER;
(**)
#if MINT
VAR dot    : BOOLEAN;
    done   : BOOLEAN;
    wres   : SIGNEDWORD;
    len    : CARDINAL;
    path01 : PathName;
    path02 : PathName;
#endif
BEGIN
#if MINT
 IF MiNT >= 9 THEN
   UnixToDos(lname, path01, dot, done);
   IF NOT done THEN
     RETURN(-1);
   END;

   Freadlink(PATHMAX+1, ADR(path02), ADR(path01), wres);
   IF (wres < eOK) AND (wres <> eRANGE) THEN
     errno := INT(wres);
     RETURN(-1);
   ELSE
     DosToUnix(path02, path02, len);
     AssignM2ToC(path02, bufsiz, buf);
     IF len >= VAL(CARDINAL,bufsiz) THEN
       RETURN(INT(bufsiz));
     ELSE
       RETURN(CAST(INTEGER,len));
     END;
   END;
 END;
#endif
 errno := ENOSYS;
 RETURN(-1);
END readlink;

(*--------------------------------------------------------------------------*)

PROCEDURE rename ((* EIN/ -- *) REF old : ARRAY OF CHAR;
                  (* EIN/ -- *) REF new : ARRAY OF CHAR ): INTEGER;
(*T*)
VAR wres   : SIGNEDWORD;
    done1  : BOOLEAN;
    done2  : BOOLEAN;
    dot    : BOOLEAN;
    path01 : PathName;
    path02 : PathName;

BEGIN
 UnixToDos(old, path01, dot, done1);
 UnixToDos(new, path02, dot, done2);
 IF NOT (done1 AND done2) THEN
   RETURN(-1);
 END;

 IF EQUAL(path01, path02) THEN
   (* sonst wird die Datei nachher geloescht... *)
   RETURN(0);
 END;

 IF access(new, fOK) = 0 THEN
   (* vorhandene Zieldatei zuvor loeschen, falls nicht schreibgeschuetzt,
    * da "GEMDOS" evtl. einen doppelten Name nicht erkennt. Wenn dabei
    * allerdings ein Fehler auftritt, ist die Zieldatei verloren!
    *)

   Fdelete(ADR(path02), wres);
   IF wres < eOK THEN
     errno := INT(wres);
     RETURN(-1);
   END;
 END;

 Frename(ADR(path01), ADR(path02), wres);
 IF wres < eOK THEN
   errno := INT(wres);
   RETURN(-1);
 ELSE
   RETURN(0);
 END;
END rename;

#if MINT
(*==========================================================================*)
BEGIN (* dir *)
 MiNT := MiNTVersion();
#endif MINT
END dir.
